;*********************************************************
         TITLE "Timer Rally"
;        Date de cration    : 21/09/00
;        Date de Mise  jour : 22/09/00
;        Auteur              ; Alain POLLET
;        Cadencement         : 3,2768Mhz
;
;	V0.1	Cration du programme - Initialisation + Timer
;	V0.2	Mise en place des menus + Lecture CLAVIER
;	V0.3	Gestion du temps ES
;	V0.4	Optimisation programme <==> Suppression problmes Delta T
;		
;
;*********************************************************

        LIST    T=ON, P=16C84, R=DEC

	INCLUDE "PICREG.16C"

;*********************************************************
;        Dclaration des variables 
;
;        De 0x0c  0x2F (36 octets)
;*********************************************************
		ORG 0x0C

	INCLUDE "RALLY.H"

timer		RES	1	; Compteur de temps
LE_RI		RES	1	; Emplacement du curseur pour reglage visuel
donnees		RES	1	; donnes  crire dans l'EEPROM
TEMP_S		RES	1	; Utilis dans la macro PUSH / POP
TEMP_W		RES	1	; Utilis dans la macro PUSH / POP


;*********************************************************
;        Dclaration des paramtres constants
;*********************************************************
SECONDE		EQU	96	; Nbr d'impulsions dans timer corespondant  0.05 seconde
INIT_TIMER	EQU	20	; Nbr d'impulsions dans compteur pour 1 seconde
ADR_NB_ES	EQU	63	; Adresse en EEPROM contenant le nombre ES en mmoire



IT_DISABLE	macro
		MOVLW	B'00000000'	; Dvalides les IT
		MOVWF	INTCON		; 
		endm
IT_ENABLE	macro
		MOVLW	B'10100000'	; Valide les IT
		MOVWF	INTCON		; 
		endm

	 ORG     0x10       ; Start address for programme

	INCLUDE "LCD_mesg.16C"


;----------------------------------------------------------
;        Dbut du programme 
;----------------------------------------------------------

MAIN	CALL	INIT		; Initialisation du programme
	CALL	INIT_LCD	; Initialisation de l'cran LCD
	CALL	MES_DEP		; Affiche le menu
	IT_ENABLE
BACK_MAIN
	CALL	AFF_MNU_GENERAL	;
MAIN0
	CALL	REGENERATION
MAIN1	
	BTFSC	CLAVIER,BP1	; BP1 appui ??
	GOTO	MAIN2		; !! NON !!
	CALL	KEY_tempo
	CALL	MNU_SET
	GOTO	BACK_MAIN
MAIN2
	BTFSC	CLAVIER,BP2	; BP2 appui ??
	GOTO	MAIN4		; !! NON !!
	CALL	KEY_tempo
	CALL	MNU_DES
	GOTO	BACK_MAIN
MAIN4
	BTFSC	CLAVIER,BP4	; BP4 appui ??
	GOTO	MAIN0		; !! NON !!
	CALL	KEY_tempo
	CALL	MNU_MEMO
	GOTO	BACK_MAIN

;----------------------------------------------------------
;        Initialisation du programme 
;----------------------------------------------------------
INIT     CLRWDT              ;Reset du watchdog
;------- Initialisation du PORT A en sortie
	 Bank1
         MOVLW   b'00000000' ;'0' = sortie, '1' = entre
         MOVWF	 TRISA
	 Bank0
         MOVLW   b'00000000' ;Initialisation des sorties A  0
         MOVWF   PORTA
;------- Initialisation du PORT B en sortie
	 Bank1
         MOVLW   b'11110000' ;'0' = sortie, '1' = entre
         MOVWF	 TRISB
	 Bank0
         MOVLW   b'00000000' ;Initialisation des sorties B  0
         MOVWF   PORTB
;------- Initialisation de l'Option Register
;        PS2  PS1  PS0  RTCC   WDT         B3 : PSA (Prescaler Assignement bit)
;         0    0    0   1:2    1:1              0 == RTCC ; 1 == WDT
;         0    0    1   1:4    1:2              
;         0    1    0   1:8    1:4         B4 : RTE (RTCC Signal Edge)
;         0    1    1   1:16   1:8              0 == Low-to-High ; 1 == High-to-Low
;         1    0    0   1:32   1:16             
;         1    0    1   1:64   1:32        B5 : RTS (RTCC Signal Source)
;         1    1    0   1:128  1:64             0 == Internal ; 1 == RTCC Pin
;         1    1    1   1:256  1:128            
	 Bank1
         MOVLW   b'00000111'   ;RTCC fos/4 / 256 (RTCC incrment) = 85s
         MOVWF	 OPTION_REG
;------- Initialisation des variables
	 Bank0
	 MOVLW	 SECONDE	 ; Charge la constante dans le registre timer0
	 MOVWF	 RTCC		 ; correspondant  50 ms

	 MOVLW	 INIT_TIMER	 ; Charge la constante dans la variable TIMER
	 MOVWF	 timer		 ; correspondant  1 seconde

	CLRF	ETAT

	CLRF	CLK_heu
	CLRF	CLK_min
	CLRF	CLK_sec

	CLRF	TI_min
	CLRF	TI_sec

	MOVLW	ADR_NB_ES
	CALL	Lit_eeprom
	MOVWF	ES

         RETURN            	 ;Fin de la procdure d'initialisation               

;*********************************************************
;        Procdure d'interruption
;        Appel tous les 50ms (3.2768E6/4/256/160 = 20hz) par le dbordement du TIMER0
;*********************************************************
INTMAIN BTFSC   INTCON,RTIF     ; Vrifie la provenance de l'interruption
	GOTO    INTTIMER        ; si dbordement du Timer 0 alors il y a saut
	MOVLW   B'00100000'     ; initialise l'IT
	MOVWF	INTCON          ; 
	GOTO INT_END		;

INTTIMER
	PUSH			; Save registers
	Bank0

	MOVLW   B'00100000'     ; initialise l'IT
	MOVWF   INTCON          ; 

	DECFSZ  timer,f		; Dcrmente et skip si TIMER est arriv  0
	GOTO	INT_FIN		; autrement saut  la fin

	MOVLW	INIT_TIMER	; Charge la constante dans la variable TIMER
	MOVWF	timer		; correspondant  1 seconde
;
; A partire d'ici on y passe toute les secondes
;
;	MOVLW	B'00000000'	; Interdit une autre IT
;	MOVWF	INTCON		;

; ----- INCREMENTATION HORLOGE -----------
	BSF	ETAT,CH_CLK
; ----- DECREMENTATION TEMPS IMPARTIS  -----------
	BTFSC	ETAT,EN_TI
	BSF	ETAT,CH_TI
; ----- CHRONOMETRE ACTIF  -----------
	BTFSC	ETAT,EN_ES
	BSF	ETAT,CH_ES
;
; Fin du traitement seconde
;

INT_FIN	MOVLW	SECONDE		; Charge la constante dans le timer0 correspondant  50 ms
	MOVWF	RTCC		;
	MOVLW	B'10100000'	; Autorise une nouvelle IT
	MOVWF	INTCON		; 
	POP			; Restor Regiters
INT_END	RETFIE			; Mission Accomplished, return to sender

;*********************************************************
;	Ajout des procedures propres au menu
;*********************************************************
	INCLUDE	"MENU.16C"
;*********************************************************
;	Ajout des procedures propres  la gestion de l'cran LCD
;*********************************************************
	INCLUDE	"LCD_4b.16C"
;*********************************************************
;        Ajout des procedures propres  la gestion de EEPROM interne
;*********************************************************
	INCLUDE "EEPROM.16C"
;*********************************************************
;        Point d'entre au RESET
;*********************************************************
         ORG     PIC84       ; Reset Vector
         GOTO    MAIN        ; Programme principal

;*********************************************************
;        Point d'entre a une Interruption 
;*********************************************************
	 ORG	 INTVEC	     ; Interrupt Vector
	 GOTO	 INTMAIN     ; Call main interupt routine

;*********************************************************
;*********************************************************
	 ORG     0x2000       ; Chip ID Data
	 DW      0x21,0x09
;*********************************************************
	 ORG     0x2007       ; Configuration Fuses
	 DW      B'00011001'
;*********************************************************
	 ORG     0x2100       ; Internal Data EEPROM Memory 
	 DW      0x0A,0x0B,0x15,0x16,0x20,0x21,0x2B,0x2C	;0
	 DW	 0x32,0x37,0x3C,0x42,0x46,0x4D,0x50,0x58	;8
	 DW	 0x5A,0x63,0x0C,0x22,0x00,0x00,0x00,0x00	;16
	 DW	 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;24
	 DW	 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;32
	 DW	 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;40
	 DW	 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00	;48
	 DW	 0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01	;56
;*********************************************************

;***** END Main Program
	end

